package org.mesopotamia.lang.java;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.MesopotamiaRuntimeException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.PrimitiveTypeInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;

public class AdditiveExpression extends ShiftExpression {

	private static final String JAVA_LANG_STRING = "java.lang.String";

	public AdditiveExpression(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes

	}

	public TypeSpecInfo getTypeSpecInfo() {
		try {
			LanguageElement firstOperand = getChild(0);
			TypeSpecInfo ts1 = firstOperand instanceof Expression ? ((Expression) firstOperand).getTypeSpecInfo() : null;
			LanguageElement secondOperand = getChild(1);			
			TypeSpecInfo ts2 = secondOperand instanceof Expression ? ((Expression) secondOperand).getTypeSpecInfo() : null;
			if (ts1==null) {
				return ts2;
			}
			if (ts2==null) {
				return ts1.getDimensions()==0 ? ts1 : null;
			}
			if (ts1.getDimensions()==0 
					&& ts2.getDimensions()==0 
					&& ts1.getTypeInfo() instanceof PrimitiveTypeInfo 
					&& ts2.getTypeInfo() instanceof PrimitiveTypeInfo) {
				if (ts1.getTypeInfo().isKindOf(ts2.getTypeInfo().getFcn())) {
					return ts2;
				}
				return ts1;
			}
			
			if (this instanceof Plus) {
				if (ts1.getDimensions()==0) {
					TypeInfo ti1 = ts1.getTypeInfo();
					if (ti1!=null && JAVA_LANG_STRING.equals(ti1.getFcn())) {
						return ts1;
					}
				}
				if (ts2.getDimensions()==0) {
					TypeInfo ti2 = ts2.getTypeInfo();
					if (ti2!=null && JAVA_LANG_STRING.equals(ti2.getFcn())) {
						return ts2;
					}
				}
			}
			
			return null;
		} catch (MesopotamiaException e) {
			throw new MesopotamiaRuntimeException(e);
		}
	}
}
