/*
  * mesopotamia-java @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */

package org.mesopotamia.lang.java;

import java.io.FilterReader;
import java.io.IOException;
import java.io.Reader;

/**
 * There is a bug/problem in java.g - it hangs on SL_COMMENT at the end of file.
 * This class virtually adds '\n' characted at the end of file.
 * 
 * @author Pavel Vlasov
 * @version $Revision: 1.1 $
 */
public class CheatingFilterReader extends FilterReader {
	private boolean cheated = false;

	/** Creates a new instance of CheatingFilterReader */
	public CheatingFilterReader(Reader in) {
		super(in);
	}

	public int read() throws IOException {
		int ret = super.read();
		if (ret == -1 && !cheated) {
			cheated = true;
			ret = '\n';
		}

		return ret;
	}

	// TODO Analyze whether is SL_COMMENT and cheat only in this case.
	public int read(char[] cbuf, int off, int len) throws IOException {
		int ret = super.read(cbuf, off, len);
		if (ret != len && !cheated) {
			cheated = true;
			cbuf[ret] = '\n';
			ret++;
		}
		return ret;
	}

	public boolean ready() throws IOException {
		return super.ready() || !cheated;
	}

	public void reset() throws IOException {
		super.reset();
		cheated = false;
	}
}
