package org.mesopotamia.lang.java;

import java.util.ArrayList;
import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

public class ClassDefinition extends ClassOrInterfaceDefinition implements Statement {

	public ClassDefinition(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		Identifier ext = selectSingleElement(Identifier.class, "EXTENDS_CLAUSE/*");
		if (ext!=null) {
			Extends = ext.getPath();
		}
		
		for(Identifier impl: select(Identifier.class, "IMPLEMENTS_CLAUSE/*")) {
			Implements.add(impl.getPath());
		}

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setAttribute(holder, "Extends", Extends);
		setElement(holder, "Implements", Implements);
	}

	// Attributes
	private String Extends;

	private List<String> Implements = new ArrayList<String>();

	// Accessors
	public String getExtends() {
		return Extends;
	}

	public List<String> getImplements() {
		return Implements;
	}
	
	@Override
	protected boolean isKindOfInternal(String superFcn) {
		if (super.isKindOfInternal(superFcn)) {
			return true;
		}
		
		Scope scope = getEnclosingScope();
		if (scope!=null) {
			String sc = getExtends();
			if (sc!=null) {
				TypeInfo sti = scope.findType(sc);
				if (sti!=null && !fcn.equals(sti.getFcn()) && sti.isKindOf(superFcn)) {
					return true;
				}
			}
			
			for (String si: getImplements()) {
				TypeInfo sti = scope.findType(si);
				if (sti!=null && sti.isKindOf(superFcn)) {
					return true;
				}
			}
		}
		
		return false;
	}

	@Override
	protected TypeInfo findNestedTypeInternal(String name) {
		TypeInfo ret = super.findNestedTypeInternal(name);
		if (ret!=null) {
			return ret;
		}
		
		Scope ec = getEnclosingScope();
		if (ec!=null) {
			if (getExtends()!=null) {
				TypeInfo spr = ec.findType(getExtends());
				if (spr!=null) {
					ret = spr.findNestedType(name);
					if (ret!=null) {
						return ret;
					}
				}
			}
			
			for (String cInterface: getImplements()) {
				TypeInfo spr = ec.findType(cInterface);
				if (spr!=null) {
					ret = spr.findNestedType(name);
					if (ret!=null) {
						return ret;
					}
				}
			}
		}
		return null;
	}
	
	@Override
	protected MethodInfo findTypeMethodInternal(String name, String[] parameterTypes) {
		MethodInfo candidate = super.findTypeMethodInternal(name, parameterTypes);
		
		Scope ec = getEnclosingScope();
		if (ec!=null) {
			if (getExtends()!=null) {
				TypeInfo spr = ec.findType(getExtends());
				if (spr!=null) {
					candidate = selectMoreSpecific(candidate, spr.findTypeMethod(name, parameterTypes));
				}
			}
			
			for (String cInterface: getImplements()) {
				TypeInfo spr = ec.findType(cInterface);
				if (spr!=null) {
					candidate = selectMoreSpecific(candidate, spr.findTypeMethod(name, parameterTypes));
				}
			}
		}
		return candidate;
	}
	
	@Override
	protected VariableInfo findTypeVariableInternal(String name) {
		VariableInfo ret = super.findTypeVariableInternal(name);
		if (ret!=null) {
			return ret;
		}
		
		Scope ec = getEnclosingScope();
		if (ec!=null) {
			if (getExtends()!=null) {
				TypeInfo spr = ec.findType(getExtends());
				if (spr!=null) {
					ret = spr.findTypeVariable(name);
					if (ret!=null) {
						return ret;
					}
				}
			}
			
			for (String cInterface: getImplements()) {
				TypeInfo spr = ec.findType(cInterface);
				if (spr!=null) {
					ret = spr.findTypeVariable(name);
					if (ret!=null) {
						return ret;
					}
				}
			}
		}
		return null;
	}
	
}
