package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

@SuppressWarnings("unchecked")
public class ClassOrInterfaceDefinition extends TypeDefinition {

	public ClassOrInterfaceDefinition(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		if (language.supportsTokenName("TYPE_PARAMETERS")) {
			TypeParameters = select(TypeParameter.class, "TYPE_PARAMETERS/*");
		}
	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "TypeParameters", TypeParameters);
	}

	// Attributes
	private List<TypeParameter> TypeParameters = emptyList;

	// Accessors
	public List<TypeParameter> getTypeParameters() {
		return TypeParameters;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(TypeParameters, false).accept(visitor);
	}
	
	@Override
	public TypeInfo findType(String name) {
		for (TypeParameter tp: getTypeParameters()) {
			if (tp.getName().equals(name)) {
				return tp;
			}
		}
		
		return super.findType(name);
	}
	
	

}
