package org.mesopotamia.lang.java;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

public class Code extends Field implements Scope {

	public Code(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		Statements = select(Statement.class, "SLIST/*");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "Statements", Statements);
	}

	// Attributes
	private List<Statement> Statements;

	// Accessors
	public List<Statement> getStatements() {
		return Statements;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(Statements, false).accept(visitor);
	}

	private Scope enclosingScope;
	protected Map<String, VariableInfo> variables;
	private Map<String, TypeDefinition> types;
	private boolean scopeSet;

	public MethodInfo findMethod(String name, String[] parameterTypes) {
		if (!scopeSet) {
			setScope();
		}
		
		return enclosingScope == null ? null : enclosingScope.findMethod(name, parameterTypes);
	}

	protected synchronized void setScope() {
		if (!scopeSet) {
			scopeSet = true;			
		}
		variables = new HashMap<String, VariableInfo>();
		types = new HashMap<String, TypeDefinition>();
		enclosingScope = findParent(Scope.class);
		
		for (Statement child: getStatements()) {
			if (child instanceof VariableDefinition) {
				VariableDefinition vd = (VariableDefinition) child;
				variables.put(vd.getName(), vd);
			} else if (child instanceof TypeDefinition) {
				TypeDefinition td = (TypeDefinition) child;
				types.put(td.getName(), td);
			}
		}
	}

	public TypeInfo findType(String name) {
		if (!scopeSet) {
			setScope();
		}
		
		TypeDefinition td = types.get(name);
		if (td!=null) {
			return td;
		}
		
		return enclosingScope == null ? null : enclosingScope.findType(name);
	}

	public VariableInfo findVariable(String name) {
		if (!scopeSet) {
			setScope();
		}
		
		VariableInfo vd = variables.get(name);
		if (vd!=null) {
			return vd;
		}
		
		return enclosingScope == null ? null : enclosingScope.findVariable(name);
	}

}
