package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

public class CompilationUnit extends JavaLanguageElement {

	public CompilationUnit(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		PackageName = selectSingleElementText(PackageDefinition.class, "PACKAGE_DEF");
		Imports = selectText(ImportDefinition.class, "IMPORT|STATIC_IMPORT");
		Types = select(TypeDefinition.class, "#org.mesopotamia.lang.java.TypeDefinition");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setAttribute(holder, "PackageName", PackageName);
		setElement(holder, "Imports", Imports);
		setElement(holder, "Types", Types);
	}

	// Attributes
	private String PackageName;

	private List<String> Imports;

	private List<TypeDefinition> Types;

	// Accessors
	public String getPackageName() {
		return PackageName;
	}

	public List<String> getImports() {
		return Imports;
	}

	public List<TypeDefinition> getTypes() {
		return Types;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(Types, false).accept(visitor);
	}

}
