package org.mesopotamia.lang.java;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;

import biz.hammurapi.util.VisitableBase;
import biz.hammurapi.util.Visitor;

public class ConditionalExpression extends AssignmentExpression implements AnnotationMemberValueInitializer {
	
	/**
	 * Represents ternary expression.
	 * @author Pavel
	 *
	 */
	public class TernaryExpression extends VisitableBase {
		
		private Expression condition;
		private Expression truePart;
		private Expression falsePart;
		
		TernaryExpression(Expression condition, Expression truePart, Expression falsePart) {
			super();
			this.condition = condition;
			this.truePart = truePart;
			this.falsePart = falsePart;
		}
		
		protected void acceptChildren(Visitor visitor) {
			object2visitor(condition, visitor);
			object2visitor(truePart, visitor);
			object2visitor(falsePart, visitor);
		}
		
		public Expression getCondition() {
			return condition;
		}
		
		public Expression getTruePart() {
			return truePart;
		}
		
		public Expression getFalsePart() {
			return falsePart;
		}
		
	}
	
	private TernaryExpression ternaryExpression;

	public ConditionalExpression(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		if (this.getClass() == ConditionalExpression.class) {
			ternaryExpression = new TernaryExpression((Expression) getChild(0), (Expression) getChild(1), (Expression) getChild(2));
		}
	}
	
	public TernaryExpression getTernaryExpression() {
		return ternaryExpression;
	}
	
	protected void acceptChildren(Visitor visitor) {
		if (this.getClass() == ConditionalExpression.class) {		
			object2visitor(ternaryExpression, visitor);
		} else {
			super.acceptChildren(visitor);
		}
	}

	public TypeSpecInfo getTypeSpecInfo() {
		if (ternaryExpression!=null) {
			return ternaryExpression.getFalsePart().getTypeSpecInfo();
		}
		return super.getTypeSpecInfo();
	}

	
}
