package org.mesopotamia.lang.java;

import java.util.Iterator;
import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Reference;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

@SuppressWarnings("unchecked")
public class ExplicitSuperConstructorInvocation extends PostfixExpression implements Reference<MethodInfo> {

	public ExplicitSuperConstructorInvocation(NodeData xData, Class<?> context,
			Scan scan, RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		if (language.supportsTokenName("TYPE_ARGUMENTS")) {
			TypeArguments = select(TypeArgument.class, "TYPE_ARGUMENTS/TYPE_ARGUMENT");
		}
		Arguments = select(Expression.class, "ELIST/*");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "TypeArguments", TypeArguments);
		setElement(holder, "Arguments", Arguments);
	}

	// Attributes
	private List<TypeArgument> TypeArguments = emptyList;

	private List<Expression> Arguments;

	// Accessors
	public List<TypeArgument> getTypeArguments() {
		return TypeArguments;
	}

	public List<Expression> getArguments() {
		return Arguments;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(TypeArguments, false).accept(visitor);
		new CollectionVisitable(Arguments, false).accept(visitor);
	}

	@Override
	public TypeSpecInfo getTypeSpecInfo() {
		return new TypeSpecInfo() {

			public int getDimensions() {
				return 0;
			}

			public TypeInfo getTypeInfo() {
				return Void.VOID_INFO;
			}
			
		};
	}

	/**
	 * Returns constructor info.
	 */
	public MethodInfo getInfo() {		
		List<Expression> args = getArguments();
		String[] argumentTypes = new String[args.size()];
		Iterator<Expression> it = args.iterator();
		for (int i=0; it.hasNext(); ++i) {
			argumentTypes[i] = TypeSpecification.toString(it.next().getTypeSpecInfo());
		}
		
		ClassDefinition cd=findParent(ClassDefinition.class);
		if (cd==null) {
			return null;
		}
		
		String ext = cd.getExtends();
		if (ext == null) {
			return null;
		}
		
		Scope scope = cd.findParent(Scope.class);
		if (scope == null) {
			return null;
		}
		
		TypeInfo ti = scope.findType(ext);
		return ti==null ? null : ti.findConstructor(argumentTypes);		
	}
		
}
