package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

@SuppressWarnings("unchecked")
public class Field extends JavaLanguageElement {

	public Field(
			NodeData xData, 
			Class<?> context, 
			Scan scan,
			RepositoryLanguage language, 
			Object environment) throws MesopotamiaException {
		
		super(xData, context, scan, language, environment);

		// Select attributes
		if (language.supportsTokenName("ANNOTATION")) {
			Modifiers = selectText(Modifier.class, "MODIFIERS/!ANNOTATION");
			Annotations = select(Annotation.class, "MODIFIERS/ANNOTATION");
		} else {
			Modifiers = selectText(Modifier.class, "MODIFIERS/*");			
		}
	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "Modifiers", Modifiers);
		setElement(holder, "Annotations", Annotations);
	}

	// Attributes
	private List<String> Modifiers;

	private List<Annotation> Annotations = emptyList;

	// Accessors
	public List<String> getModifiers() {
		return Modifiers;
	}

	public List<Annotation> getAnnotations() {
		return Annotations;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(Annotations, false).accept(visitor);
	}

}
