package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

public class For extends JavaLanguageElement implements Statement, Scope {

	public For(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		Initializers = select(JavaLanguageElement.class, "FOR_INIT/*");
		Condition = (Expression) selectSingleElement(Expression.class, "FOR_CONDITION/*");
		Iterator = select(Expression.class, "FOR_ITERATOR/ELIST/*");
		if (language.supportsTokenName("FOR_EACH_CLAUSE")) {
			ForEachClause = selectSingleElement(ForEachClause.class, "FOR_EACH_CLAUSE");
		}
		Statement = (Statement) selectSingleElement(Statement.class, "#org.mesopotamia.lang.java.Statement");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "Initializers", Initializers);
		setElement(holder, "Condition", Condition);
		setElement(holder, "Iterator", Iterator);
		setElement(holder, "ForEachClause", ForEachClause);
		setElement(holder, "Statement", Statement);
	}

	// Attributes
	private List<JavaLanguageElement> Initializers;

	private Expression Condition;

	private List<Expression> Iterator;

	private ForEachClause ForEachClause;

	private Statement Statement;

	// Accessors
	public List<JavaLanguageElement> getInitializers() {
		return Initializers;
	}

	public Expression getCondition() {
		return Condition;
	}

	public List<Expression> getIterator() {
		return Iterator;
	}

	public ForEachClause getForEachClause() {
		return ForEachClause;
	}

	public Statement getStatement() {
		return Statement;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(Initializers, false).accept(visitor);
		if (Condition instanceof Visitable) {
			((Visitable) Condition).accept(visitor);
		}
		new CollectionVisitable(Iterator, false).accept(visitor);
		if (ForEachClause instanceof Visitable) {
			((Visitable) ForEachClause).accept(visitor);
		}
		if (Statement instanceof Visitable) {
			((Visitable) Statement).accept(visitor);
		}
	}

	public MethodInfo findMethod(String name, String[] parameterTypes) {
		Scope ec = findParent(Scope.class);
		return ec==null ? null : ec.findMethod(name, parameterTypes);
	}

	public TypeInfo findType(String name) {
		Scope ec = findParent(Scope.class);
		return ec==null ? null : ec.findType(name);
	}

	public VariableInfo findVariable(String name) {
		Scope ec = findParent(Scope.class);
		for (JavaLanguageElement initializer: getInitializers()) {
			if (initializer instanceof VariableDefinition) {				
				VariableDefinition ret = (VariableDefinition) initializer;
				if (ret.getName().equals(name)) {
					return ret;
				}
				
			}
		}
		
		if (getForEachClause()!=null) {
			ParameterDefinition pd = getForEachClause().getParameterDefinition();
			if (pd.getName().equals(name)) {
				return pd;
			}
		}
		return ec==null ? null : ec.findVariable(name);
	}

}
