package org.mesopotamia.lang.java;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

public class Handler extends JavaLanguageElement implements Scope {

	public Handler(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		ParameterDefinition = selectSingleElement(ParameterDefinition.class,
				"PARAMETER_DEF");
		CompoundStatement = selectSingleElement(CompoundStatement.class, "SLIST");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "ParameterDefinition", ParameterDefinition);
		setElement(holder, "CompoundStatement", CompoundStatement);
	}

	// Attributes
	private ParameterDefinition ParameterDefinition;

	private CompoundStatement CompoundStatement;

	// Accessors
	public ParameterDefinition getParameterDefinition() {
		return ParameterDefinition;
	}

	public CompoundStatement getCompoundStatement() {
		return CompoundStatement;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		if (ParameterDefinition instanceof Visitable) {
			((Visitable) ParameterDefinition).accept(visitor);
		}
		if (CompoundStatement instanceof Visitable) {
			((Visitable) CompoundStatement).accept(visitor);
		}
	}

	public MethodInfo findMethod(String name, String[] parameterTypes) {
		Scope ec = findParent(Scope.class);
		return ec == null ? null : ec.findMethod(name, parameterTypes);
	}

	public TypeInfo findType(String name) {
		Scope ec = findParent(Scope.class);
		return ec == null ? null : ec.findType(name);
	}

	public VariableInfo findVariable(String name) {
		if (name.equals(ParameterDefinition.getName())) {
			return ParameterDefinition;
		}
		
		Scope ec = findParent(Scope.class);
		return ec == null ? null : ec.findVariable(name);
	}

}
