package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.MesopotamiaNode;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.Info;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.mesopotamia.util.UnmodifiableConvertingList;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

public class Identifier extends PrimaryExpression implements Type {

	public Identifier(NodeData xData, Class<?> context, Scan scan,	RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);
	}
	
	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "TypeArguments", TypeArguments);
	}

	// Attributes
	private UnmodifiableConvertingList<TypeArgument> TypeArguments;

	/**
	 * Full text with dots.
	 */
	public String getPath() {
		if (providerHandle==null) {
			return nodeText(getNode());
		}
		
		MesopotamiaNode providerNode = getSourceUnit().getSyntaxTree().findNode(providerHandle.getId());
		return nodeText("DOT".equals(providerNode.getParent().getTypeName()) ? providerNode.getParent() : providerNode); 
	}

	// Accessors
	public List<TypeArgument> getTypeArguments() {
		return TypeArguments;
	}

	protected void acceptChildren(Visitor visitor) {
		// super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(TypeArguments, false).accept(visitor);
	}
		
	/**
	 * Constructs string from DOT tree.
	 * @param mn
	 * @return
	 */
	public static String nodeText(MesopotamiaNode mn) {
		if ("DOT".equals(mn.getTypeName()) && mn.getChildren().size()==2) {
			return nodeText((MesopotamiaNode) mn.getChildren().get(0))+"."+nodeText((MesopotamiaNode) mn.getChildren().get(1));
		}
		
		if ("IDENT".equals(mn.getTypeName())) {
			return mn.getText();
		}
		
		if (mn.getChildren().size()==1) {
			return nodeText((MesopotamiaNode) mn.getChildren().get(0));
		}
		
		return null;
	}

	public Info getInfo() {
		return getInfo(findParent(Scope.class));
	}
	
	Info getInfo(Scope es) {
		if (getText()==null) {
			return null;
		}
		LanguageElement provider = getProvider();
		if (provider == null) {
			if (es!=null) {
				VariableInfo ret = es.findVariable(getText());
				if (ret!=null) {
					return ret;
				}
				return es.findType(getText());
			}
		} else {
			TypeSpecInfo typeSpecInfo = null;
			if (provider instanceof Identifier) {
				typeSpecInfo = ((Identifier) provider).getTypeSpecInfo(es);
			} else if (provider instanceof MethodCall) {
				typeSpecInfo = ((MethodCall) provider).getTypeSpecInfo(es);
			} else if (provider instanceof Expression) {
				 typeSpecInfo = ((Expression) provider).getTypeSpecInfo();
			}
			
			if (typeSpecInfo!=null) {
				TypeInfo ti = TypeSpecification.toTypeInfo(typeSpecInfo, es);
				if (ti!=null) {					
					TypeInfo type = ti.findNestedType(getText());
					if (type!=null) {
						return type;
					}
					return ti.findTypeVariable(getText());
				}
			}
			
			// FCN?
			if (es!=null) {
				String path = getPath();
				VariableInfo vi = es.findVariable(path);
				if (vi!=null) {
					return vi;
				} 
				
				TypeInfo ti = es.findType(path);
				if (ti!=null) {
					return ti;
				}
				
			}				
		}
		return null;
	}
	
	@Override
	public TypeSpecInfo getTypeSpecInfo() {
		return getTypeSpecInfo(findParent(Scope.class));
	}

	public TypeSpecInfo getTypeSpecInfo(Scope scope) {
		final Info info = getInfo(scope);
		if (info instanceof TypeInfo) {
			return new TypeSpecInfo() {

				public int getDimensions() {
					return 0;
				}

				public TypeInfo getTypeInfo() {
					return (TypeInfo) info;
				}
				
			};
		} else if (info instanceof VariableInfo) {
			return ((VariableInfo) info).getTypeSpecification();
		}
		return null;
	}
	
}
