package org.mesopotamia.lang.java;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.MesopotamiaRuntimeException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

public class IndexOperation extends PostfixExpression {

	public IndexOperation(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		Index = (Expression) selectSingleElement(Expression.class, "EXPR");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "Index", Index);
	}

	// Attributes
	private Expression Index;

	// Accessors
	public Expression getIndex() {
		return Index;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		if (Index instanceof Visitable) {
			((Visitable) Index).accept(visitor);
		}
	}

	public TypeSpecInfo getTypeSpecInfo() {
		try {
			LanguageElement firstOperand = getChild(0);
			final TypeSpecInfo ts =  firstOperand instanceof Expression ? ((Expression) firstOperand).getTypeSpecInfo() : null;
			return ts==null || ts.getDimensions()==0 ? null : new TypeSpecInfo() {

				public int getDimensions() {
					return ts.getDimensions()-1;
				}

				public TypeInfo getTypeInfo() {
					return ts.getTypeInfo();
				}
				
			};
		} catch (MesopotamiaException e) {
			throw new MesopotamiaRuntimeException(e);
		}
	}
}
