package org.mesopotamia.lang.java;

import java.util.ArrayList;
import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitor;

public class InterfaceDefinition extends ClassOrInterfaceDefinition {

	public InterfaceDefinition(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		
		for(Identifier ext: select(Identifier.class, "EXTENDS_CLAUSE/*")) {
			Extends.add(ext.getPath());
		}

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "Extends", Extends);
	}

	// Attributes
	private List<String> Extends = new ArrayList<String>();

	// Accessors
	public List<String> getExtends() {
		return Extends;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
	}
	
	@Override
	protected boolean isKindOfInternal(String superFcn) {
		if (super.isKindOfInternal(superFcn)) {
			return true;
		}
		
		Scope scope = getEnclosingScope();
		if (scope!=null) {
			for (String si: getExtends()) {
				TypeInfo sti = scope.findType(si);
				if (sti!=null && sti.isKindOf(superFcn)) {
					return true;
				}
			}
		}
		
		return false;
	}


	@Override
	protected TypeInfo findNestedTypeInternal(String name) {
		TypeInfo ret = super.findNestedTypeInternal(name);
		if (ret!=null) {
			return ret;
		}
		
		Scope ec = getEnclosingScope();
		if (ec!=null) {			
			for (String cInterface: getExtends()) {
				TypeInfo spr = ec.findType(cInterface);
				if (spr!=null) {
					ret = spr.findNestedType(name);
					if (ret!=null) {
						return ret;
					}
				}
			}
		}
		return null;
	}
	
	@Override
	protected MethodInfo findTypeMethodInternal(String name, String[] parameterTypes) {
		MethodInfo candidate = super.findTypeMethodInternal(name, parameterTypes);
		
		Scope ec = getEnclosingScope();
		if (ec!=null) {
			for (String cInterface: getExtends()) {
				TypeInfo spr = ec.findType(cInterface);
				if (spr!=null) {
					candidate = selectMoreSpecific(candidate, spr.findTypeMethod(name, parameterTypes));
				}
			}
		}
		return candidate;
	}
	
	@Override
	protected VariableInfo findTypeVariableInternal(String name) {
		VariableInfo ret = super.findTypeVariableInternal(name);
		if (ret!=null) {
			return ret;
		}
		
		Scope ec = getEnclosingScope();
		if (ec!=null) {
			for (String cInterface: getExtends()) {
				TypeInfo spr = ec.findType(cInterface);
				if (spr!=null) {
					ret = spr.findTypeVariable(name);
					if (ret!=null) {
						return ret;
					}
				}
			}
		}
		return null;
	}
}
