package org.mesopotamia.lang.java;

import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.mesopotamia.MesopotamiaRuntimeException;
import org.mesopotamia.lang.java.javadoc.JavaDocLexer;
import org.mesopotamia.lang.java.javadoc.JavaDocParser;
import org.mesopotamia.lang.java.javadoc.JavaDocTokenTypes;
import org.mesopotamia.lang.java.javadoc.Tag;

import antlr.ASTFactory;
import antlr.collections.AST;

/**
 * 
 * @author Pavel
 *
 */
public class JavaDocComment extends MultiLineComment {

	private Exception exception;
	private biz.hammurapi.antlr.AST ast;
	private List<Tag> tags = new ArrayList<Tag>();
	private String stringDescription;
	private List<String> description = new ArrayList<String>();

	public JavaDocComment(int line, int col, String text) {
		super(line, col, text);
		try {
			StringReader r=new StringReader(text);
			JavaDocLexer lexer=new JavaDocLexer(r);
			lexer.setTokenObjectClass(biz.hammurapi.antlr.Token.class.getName());
			JavaDocParser parser=new JavaDocParser(lexer);
			ASTFactory astFactory = new ASTFactory();
			astFactory.setASTNodeClass(biz.hammurapi.antlr.AST.class);
			parser.setASTFactory(astFactory);
			parser.specificationBlock();
			this.ast=(biz.hammurapi.antlr.AST) parser.getAST();
//			ast.showWithSiblings("JavaDoc", parser.getTokenNames());
			
			for (AST node=this.ast; node!=null; node=node.getNextSibling()) {
				switch (node.getType()) {
					case JavaDocTokenTypes.DESCRBLOCK:
						readDescription(node);
						break;
					case JavaDocTokenTypes.TAGBLOCK:
						tags.add(new Tag(node));
						break;
					default:
						throw new MesopotamiaRuntimeException("Invalid javadoc token type: "+node.getType()+" for "+text); 
					
				}
			}
		} catch (Exception e) {
			this.exception=e;
		}
		
	}
	
	/**
	 * @param ast
	 */
	private void readDescription(AST ast) {
		StringBuffer sd=new StringBuffer();
		for (AST node=ast.getFirstChild(); node!=null; node=node.getNextSibling()) {
			description  .add(node.getText());
			sd.append(node.getText());
			if (node.getNextSibling()!=null) {
				sd.append(" ");
			}
		}
		stringDescription=sd.toString();
	}
	
	
	public Exception getException() {
		return exception;
	}

	public List<String> getDescription() {
		return Collections.unmodifiableList(description);
	}
	
	public String getDescriptionAsString() {
		return stringDescription;
	}
	
	public List<Tag> getTags() {
		return Collections.unmodifiableList(tags);
	}
}
