/*
 * mesopotamia-java @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.LanguageElementHandle;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;

import biz.hammurapi.util.VisitableBase;
import biz.hammurapi.util.Visitor;

/**
 * Base class for all language elements for Java language
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class JavaLanguageElement extends LanguageElement {

	public static final String JAVA_LANG_OBJECT = "java.lang.Object";

	public JavaLanguageElement(
			NodeData xData, 
			Class<?> context, 
			Scan scan,
			RepositoryLanguage language, 
			Object environment) {
		super(xData, context, scan, language, environment);
		if (xData instanceof MesopotamiaJavaNode) {
			comments = ((MesopotamiaJavaNode) xData).getComments();
		}
	}
	
	private List<Comment> comments;
	
	/**
	 * @return Comments before this element, if any.
	 */
	public List<Comment> getComments() {
		return comments;
	}

	protected LanguageElementHandle providerHandle;
	
	public void setProviderHandle(LanguageElementHandle providerHandle) {
		this.providerHandle = providerHandle;
	}
	
	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		VisitableBase.object2visitor(getProvider(), visitor);
	}	
	
	/**
	 * Provider for this language element, i.e. what comes before DOT.
	 * @return
	 */
	public LanguageElement getProvider() {
		return (LanguageElement) (providerHandle == null ? null : getScan().getLanguageElement(providerHandle));
	}
	
	protected String fcn;
	protected boolean fcnCalculated;
		
	/**
	 * Helper method for giving FCN's to anonymous and local classes.
	 * @return
	 */
	protected String getFcn() {
		if (!fcnCalculated) {
			LanguageElement parent = getParent();
			if (parent instanceof JavaLanguageElement) {
				fcn = ((JavaLanguageElement) parent).getFcn()+"."+getId();
			} else {
				fcn = String.valueOf(getId());
			}
			fcnCalculated = true;
		}
		return fcn;
	}
}
