package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Reference;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

/**
 * Method call implements scope as a shortcut
 * @author Pavel
 *
 */
public class MethodCall extends PostfixExpression implements Reference<MethodInfo> {

	private Identifier nameElement;

	public MethodCall(NodeData xData, Class<?> context, Scan scan, RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);
		nameElement = (Identifier) getChild(0);
		Arguments = select(Expression.class, "ELIST/*");
	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "name", nameElement);
		setElement(holder, "Arguments", Arguments);
		setElement(holder, "Provider", getProvider());
	}

	// Attributes
	private List<Expression> Arguments;

	// Accessors
	public String getName() {
		return nameElement.getText();
	}

	public LanguageElement getProvider() {
		return (LanguageElement) (nameElement.providerHandle == null ? null : getScan().getLanguageElement(nameElement.providerHandle));
	}

	public List<Expression> getArguments() {
		return Arguments;
	}

	protected void acceptChildren(Visitor visitor) {
		nameElement.accept(visitor);
		new CollectionVisitable(Arguments, false).accept(visitor);
		object2visitor(getProvider(), visitor);
	}

	public List<TypeArgument> getTypeArguments() {
		return nameElement.getTypeArguments();
	}
	
	public MethodInfo getInfo() {
		return getInfo(findParent(Scope.class));
	}
		
	public MethodInfo getInfo(Scope es) {
		
		List<Expression> args = getArguments();
		String[] argTypes = new String[args.size()];
		int idx=0;
		for (Expression arg: args) {
			argTypes[idx++] = TypeSpecification.toString(arg.getTypeSpecInfo());
		}
		
		LanguageElement provider = getProvider();
		if (provider == null) {
			if (es!=null) {
				MethodInfo ret = es.findMethod(getName(), argTypes);
				if (ret!=null) {
					return ret;
				}
			}
		} else {
			TypeSpecInfo typeSpecInfo = null;
			if (provider instanceof Identifier) {
				typeSpecInfo = ((Identifier) provider).getTypeSpecInfo(es);
			} else if (provider instanceof MethodCall) {
				typeSpecInfo = ((MethodCall) provider).getTypeSpecInfo(es);
			} else if (provider instanceof Expression) {
				 typeSpecInfo = ((Expression) provider).getTypeSpecInfo();
			}
			
			if (typeSpecInfo!=null) {
				TypeInfo ti = TypeSpecification.toTypeInfo(typeSpecInfo, es);
				if (ti!=null) {					
					MethodInfo ret = ti.findTypeMethod(getName(), argTypes);
					if (ret!=null) {
						return ret;
					}
				}
			}
		}
		return null;
	}
	
	@Override
	public TypeSpecInfo getTypeSpecInfo() {
		return getTypeSpecInfo(findParent(Scope.class));
	}

	public TypeSpecInfo getTypeSpecInfo(Scope es) {
		MethodInfo info = getInfo(es);
		return info==null ? null : info.getReturnType();
	}

}
