package org.mesopotamia.lang.java;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitor;

public class MethodDefinition extends Operation implements MethodInfo {

	public MethodDefinition(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		Name = selectSingleElementText(Identifier.class, "IDENT|DOT");
		ReturnType = selectSingleElement(TypeSpecification.class, "TYPE");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setAttribute(holder, "Name", Name);
	}

	// Attributes
	private String Name;

	private TypeSpecification ReturnType;

	// Accessors
	public String getName() {
		return Name;
	}

	public TypeSpecification getReturnType() {
		return ReturnType;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		object2visitor(ReturnType, visitor);
	}

	public String getFcn() {
		if (!fcnCalculated) {
			fcnCalculated = true;
			TypeInfo dt = getDeclaringType();
			if (dt==null) {
				return null;
			}
			StringBuffer ret = new StringBuffer();
			TypeSpecification rt = getReturnType();
			if (rt==null) {
				return null;
			}
			TypeInfo returnTypeInfo = rt.getTypeInfo();
			if (returnTypeInfo==null) {
				return null;
			}
			ret.append(returnTypeInfo.getFcn());
			for (int i=0; i<rt.getDimensions(); ++i) {
				ret.append(TypeSpecInfo.ARRAY_BRACKETS);
			}	
			ret.append(" ");
			ret.append(dt.getFcn());
			ret.append(".");
			ret.append(getName());
			ret.append("(");
			boolean first = true;
			for (TypeSpecInfo pd: getParameterTypes()) {
				if (pd==null) {
					return null;
				}
				if (first) {
					first = false;
				} else {
					ret.append(", ");
				}
				TypeInfo pTypeInfo = pd.getTypeInfo();
				if (pTypeInfo == null) {
					return null;
				}
				ret.append(pTypeInfo.getFcn());
				for (int i=0; i<pd.getDimensions(); ++i) {
					ret.append(TypeSpecInfo.ARRAY_BRACKETS);
				}
			}
			ret.append(")");
			first = true;
			for (TypeInfo et: getExceptionTypes()) {
				if (et==null) {
					return null;
				}
				if (first) {
					ret.append(" throws ");
					first = false;
				} else {
					ret.append(", ");
				}
				ret.append(et.getFcn());
			}
			fcn = ret.toString();			
		}
		return fcn;
	}

}
