package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;

public class Null extends PrimaryExpression {
	
	public static final String NULL_FCN = "(null)";

	public static final TypeInfo NULL_TYPE_INFO = new TypeInfo() {

		public MethodInfo findConstructor(String[] argumentTypes) {
			return null;
		}

		public TypeInfo findNestedType(String name) {
			return null;
		}

		public MethodInfo findTypeMethod(String name, String[] argumentTypes) {
			return null;
		}

		public VariableInfo findTypeVariable(String name) {
			return null;
		}

		/**
		 * Null is compatible with object types.
		 */
		public boolean isKindOf(String superFcn) {
			return BuiltInType.getBuiltInType(superFcn)==null;
		}

		public TypeInfo getDeclaringType() {
			return null;
		}

		public String getFcn() {
			return NULL_FCN;
		}

		@SuppressWarnings("unchecked")
		public List<String> getModifiers() {
			return emptyList;
		}

		public String getName() {
			return NULL_FCN;
		}
		
	};
	
	public static final TypeSpecInfo NULL_TYPE_SPEC_INFO = new TypeSpecInfo() {

		public int getDimensions() {
			return -1;
		}

		public TypeInfo getTypeInfo() {
			return NULL_TYPE_INFO;
		}
		
	};

	public Null(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes

	}
	
	@Override
	public TypeSpecInfo getTypeSpecInfo() {
		return NULL_TYPE_SPEC_INFO;
	}

}
