package org.mesopotamia.lang.java;

import java.util.ArrayList;
import java.util.List;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitor;

@SuppressWarnings("unchecked")
public class Operation extends Code {

	public Operation(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		if (language.supportsTokenName("TYPE_PARAMETERS")) {
			TypeParameters = select(TypeParameter.class, "TYPE_PARAMETERS/TYPE_PARAMETER");
		}
		Parameters = select(ParameterDefinition.class, "PARAMETERS/*");
		for(Identifier th: select(Identifier.class, "LITERAL_throws/*")) {
			Throws.add(th.getPath());
		}
	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setElement(holder, "TypeParameters", TypeParameters);
		setElement(holder, "Parameters", Parameters);
		setElement(holder, "Throws", Throws);
	}

	// Attributes
	private List<TypeParameter> TypeParameters = emptyList;

	private List<ParameterDefinition> Parameters;

	private List<String> Throws = new ArrayList<String>();

	// Accessors
	public List<TypeParameter> getTypeParameters() {
		return TypeParameters;
	}

	public List<ParameterDefinition> getParameters() {
		return Parameters;
	}

	public List<String> getThrows() {
		return Throws;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		new CollectionVisitable(TypeParameters, false).accept(visitor);
		new CollectionVisitable(Parameters, false).accept(visitor);
	}
	
	@Override
	protected synchronized void setScope() {
		super.setScope();
		
		for (ParameterDefinition p: getParameters()) {
			variables.put(p.getName(), p);
		}
	}
	
	@Override
	public TypeInfo findType(String name) {
		for (TypeParameter tp: getTypeParameters()) {
			if (tp.getName().equals(name)) {
				return tp;
			}
		}
		
		return super.findType(name);
	}
	
	public List<TypeSpecInfo> getParameterTypes() {
		List<TypeSpecInfo> ret = new ArrayList<TypeSpecInfo>();
		for (ParameterDefinition pd: getParameters()) {
			ret.add(pd.getTypeSpecification());
		}
		return ret;
	}

	public List<TypeInfo> getExceptionTypes() {
		List<TypeInfo> ret = new ArrayList<TypeInfo>();
		Scope ec = findParent(Scope.class);
		for (String et: getThrows()) {
			ret.add(ec==null ? null : ec.findType(et));
		}
		return ret;
	}

	public TypeInfo getDeclaringType() {
		return findParent(TypeDefinitionOrAnonymous.class);
	}
	
}
