package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.CollectionVisitable;
import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

@SuppressWarnings("unchecked")
public class ParameterDefinition extends JavaLanguageElement implements VariableInfo {

	public ParameterDefinition(
			NodeData xData,
			Class<?> context,
			Scan scan,
			RepositoryLanguage language,
			Object environment) throws MesopotamiaException {
		
		super(xData, context, scan, language, environment);

		// Select attributes
		Name = selectSingleElementText(Identifier.class, "IDENT|DOT");
		TypeSpecification = selectSingleElement(TypeSpecification.class, "TYPE");
		
		// Select attributes
		if (language.supportsTokenName("ANNOTATION")) {
			Modifiers = selectText(Modifier.class, "MODIFIERS/!ANNOTATION");
			Annotations = select(Annotation.class, "MODIFIERS/ANNOTATION");
		} else {
			Modifiers = selectText(Modifier.class, "MODIFIERS/*");			
		}
	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setAttribute(holder, "Name", Name);
		setElement(holder, "TypeSpecification", TypeSpecification);
		setElement(holder, "Modifiers", Modifiers);
		setElement(holder, "Annotations", Annotations);
	}

	// Attributes
	private String Name;

	private TypeSpecification TypeSpecification;

	private List<String> Modifiers;

	// Accessors
	/* (non-Javadoc)
	 * @see org.mesopotamia.lang.java.VariableOrParameterDefinition#getName()
	 */
	public String getName() {
		return Name;
	}

	/* (non-Javadoc)
	 * @see org.mesopotamia.lang.java.VariableOrParameterDefinition#getTypeSpecification()
	 */
	public TypeSpecification getTypeSpecification() {
		return TypeSpecification;
	}

	/* (non-Javadoc)
	 * @see org.mesopotamia.lang.java.VariableOrParameterDefinition#getModifiers()
	 */
	public List<String> getModifiers() {
		return Modifiers;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		if (TypeSpecification instanceof Visitable) {
			((Visitable) TypeSpecification).accept(visitor);
		}
		
		new CollectionVisitable(Annotations, false).accept(visitor);
	}
	
	private List<Annotation> Annotations = emptyList;

	/* (non-Javadoc)
	 * @see org.mesopotamia.lang.java.VariableOrParameterDefinition#getAnnotations()
	 */
	public List<Annotation> getAnnotations() {
		return Annotations;
	}

	public TypeInfo getDeclaringType() {
		return null;
	}

	public String getFcn() {
		return null;
	}
	
}
