package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

public class TypeParameter extends JavaLanguageElement implements TypeInfo {

	public TypeParameter(NodeData xData, Class<?> context, Scan scan,
			RepositoryLanguage language, Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);

		// Select attributes
		Name = selectSingleElementText(Identifier.class, "IDENT[0]");
		TypeBound = selectSingleElement(TypeBound.class, "#org.mesopotamia.lang.java.TypeBound");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setAttribute(holder, "Name", Name);
		setElement(holder, "TypeBound", TypeBound);
	}

	// Attributes
	private String Name;

	private TypeBound TypeBound;

	// Accessors
	public String getName() {
		return Name;
	}

	public TypeBound getTypeBound() {
		return TypeBound;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		if (TypeBound instanceof Visitable) {
			((Visitable) TypeBound).accept(visitor);
		}
	}

	public MethodInfo findTypeMethod(String name, String[] parameterTypes) {
		Scope ec = findParent(Scope.class);
		if (ec!=null) {
			TypeInfo ti = ec.findType(TypeBound==null ? JAVA_LANG_OBJECT : TypeBound.getBoundType().getPath());
			if (ti!=null) {
				return ti.findTypeMethod(name, parameterTypes);
			}
		}
		
		return null;
	}

	public TypeInfo findNestedType(String name) {
		if (TypeBound==null) {
			return null;
		}
		Scope ec = findParent(Scope.class);
		if (ec!=null) {
			TypeInfo ti = ec.findType(TypeBound.getBoundType().getPath());
			if (ti!=null) {
				return ti.findNestedType(name);
			}
		}
		
		return null;
	}

	public VariableInfo findTypeVariable(String name) {
		Scope ec = findParent(Scope.class);
		if (ec!=null) {
			TypeInfo ti = ec.findType(TypeBound==null ? JAVA_LANG_OBJECT : TypeBound.getBoundType().getPath());
			if (ti!=null) {
				return ti.findTypeVariable(name);
			}
		}
		
		return null;
	}

	public boolean isKindOf(String superFcn) {
		if (TypeBound==null) {
			return JAVA_LANG_OBJECT.equals(superFcn);
		}
		Scope ec = findParent(Scope.class);
		if (ec!=null) {
			TypeInfo ti = ec.findType(TypeBound.getBoundType().getPath());
			if (ti!=null) {
				return ti.isKindOf(superFcn);
			}
		}
		
		return false;
	}

	public String getFcn() {
		if (TypeBound==null) {
			return JAVA_LANG_OBJECT;
		}
		Scope ec = findParent(Scope.class);
		if (ec!=null) {
			Identifier boundType = TypeBound.getBoundType();
			if (boundType!=null) {
				TypeInfo ti = ec.findType(boundType.getPath());
				if (ti!=null) {
					return ti.getFcn();
				}
			}
		}
		
		return null;
	}

	public TypeInfo getDeclaringType() {
		return null;
	}

	@SuppressWarnings("unchecked")
	public List<String> getModifiers() {
		return emptyList;
	}


	public MethodInfo findConstructor(String[] parameterTypes) {
		if (TypeBound==null) {
			return null;
		}
		Scope ec = findParent(Scope.class);
		if (ec!=null) {
			TypeInfo ti = ec.findType(TypeBound.getBoundType().getPath());
			if (ti!=null) {
				return ti.findConstructor(parameterTypes);
			}
		}
		
		return null;
	}
	
}
