package org.mesopotamia.lang.java;

import java.util.List;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

public class TypeSpecification extends TypeArgument implements TypeSpecInfo {

	private int dimensions;
	private Type type;

	public TypeSpecification(
			NodeData xData, 
			Class<?> context, 
			Scan scan,
			RepositoryLanguage language, 
			Object environment) throws MesopotamiaException {
		super(xData, context, scan, language, environment);
		LanguageElement child=getChild(0);
		while ("ARRAY_DECLARATOR".equals(child.getTokenName())) {
			dimensions++;
			child=child.getChild(0);
		}
		
		type=(Type) child;
	}
	
	/**
	 * Array dimensons.
	 * @return
	 */
	public int getDimensions() {
		return dimensions;
	}
	
	public Type getSpecificationType() {
		return type;
	}

	public void toDom(Element holder) {
		super.toDom(holder);
		setAttribute(holder, "dimensions", String.valueOf(dimensions));
		setElement(holder, "type", type);
	}
	
	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		((Visitable) type).accept(visitor);
	}
	
	public TypeInfo getTypeInfo() {
		if (type instanceof BuiltInType) {
			return (TypeInfo) type.getInfo();
		}
		// Can be only identifier here.
		String typeName = ((Identifier) type).getPath();
		Scope ec = findParent(Scope.class);
		return ec==null ? null : ec.findType(typeName);
	}

	public static String toString(TypeSpecInfo typeSpecInfo) {
		if (typeSpecInfo == null) {
			return null;
		}
		TypeInfo ti = typeSpecInfo.getTypeInfo();
		if (ti==null) {
			return null;
		}				
		String tfcn = ti.getFcn();
		StringBuffer ret = new StringBuffer(tfcn);
		for (int i=0; typeSpecInfo!=Null.NULL_TYPE_SPEC_INFO && i<typeSpecInfo.getDimensions(); ++i) {
			ret.append(ARRAY_BRACKETS);
		}
		return ret.toString();
	}
	
	/**
	 * Returns type info for given type spec info taking dimensions
	 * into consideration.
	 * @param tsi
	 * @param es
	 * @return
	 */
	public static TypeInfo toTypeInfo(TypeSpecInfo tsi, Scope es) {
		if (tsi==null) {
			return null;
		}
		
		if (tsi.getDimensions()==0) {
			return tsi.getTypeInfo();
		}
		
		if (es==null) {
			return null;
		}
		
		final TypeInfo objType = es.findType(JAVA_LANG_OBJECT);
		if (objType==null) {
			return null;
		}
		
		return new TypeInfo() {

			public TypeInfo findNestedType(String name) {
				return objType.findNestedType(name);
			}

			public MethodInfo findTypeMethod(String name, String[] parameterTypes) {
				return objType.findTypeMethod(name, parameterTypes);
			}

			public VariableInfo findTypeVariable(String name) {
				if ("length".equals(name)) {
					return new VariableInfo() {

						public TypeSpecInfo getTypeSpecification() {
							return new TypeSpecInfo() {

								public int getDimensions() {
									return 0;
								}

								public TypeInfo getTypeInfo() {
									return Int.INT_INFO;												}
								
							};
						}

						public TypeInfo getDeclaringType() {
							return null;
						}

						public String getFcn() {
							return null;
						}

						public List<String> getModifiers() {
							return null;
						}

						public String getName() {
							return "length";
						}
						
					};
				}
				return objType.findTypeVariable(name);
			}

			public boolean isKindOf(String superFcn) {
				return objType.isKindOf(superFcn);
			}

			public TypeInfo getDeclaringType() {
				return null;
			}

			public String getFcn() {
				return null;
			}

			public List<String> getModifiers() {
				return null;
			}

			public String getName() {
				return null;
			}

			public MethodInfo findConstructor(String[] parameterTypes) {
				return null;
			}
			
		};
	}
	
}
