package org.mesopotamia.lang.java;

import org.mesopotamia.LanguageElement;
import org.mesopotamia.MesopotamiaException;
import org.mesopotamia.NodeData;
import org.mesopotamia.RepositoryLanguage;
import org.mesopotamia.Scan;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.VariableInfo;
import org.w3c.dom.Element;

import biz.hammurapi.util.Visitable;
import biz.hammurapi.util.Visitor;

public class VariableDefinition extends Field implements Statement, VariableInfo {

	public VariableDefinition(
			NodeData xData, 
			Class<?> context, 
			Scan scan,
			RepositoryLanguage language, 
			Object environment) throws MesopotamiaException {
		
		super(xData, context, scan, language, environment);

		// Select attributes
		Name = selectSingleElementText(Identifier.class, "IDENT|DOT");
		TypeSpecification = selectSingleElement(TypeSpecification.class, "TYPE");
		Initializer = selectSingleElement(VariableInitializer.class, "ASSIGN");

	}

	public void toDom(Element holder) {
		super.toDom(holder);

		// Serialize attributes
		setAttribute(holder, "Name", Name);
		setElement(holder, "TypeSpecification", TypeSpecification);
		setElement(holder, "Initializer", Initializer);
	}

	// Attributes
	private String Name;

	private TypeSpecification TypeSpecification;

	private VariableInitializer Initializer;

	// Accessors
	public String getName() {
		return Name;
	}

	public TypeSpecification getTypeSpecification() {
		return TypeSpecification;
	}

	public VariableInitializer getInitializer() {
		return Initializer;
	}

	protected void acceptChildren(Visitor visitor) {
		super.acceptChildren(visitor);
		// Visiting non-text attributes
		if (TypeSpecification instanceof Visitable) {
			((Visitable) TypeSpecification).accept(visitor);
		}
		if (Initializer instanceof Visitable) {
			((Visitable) Initializer).accept(visitor);
		}
	}

	public String getFcn() {
		LanguageElement parent = getParent();
		if (parent instanceof TypeDefinition) {
			String pFcn = ((TypeDefinition) parent).getFcn();
			if (pFcn!=null) {
				return pFcn+"."+getName();
			}
		}
		return null;
	}

	public TypeInfo getDeclaringType() {
		LanguageElement parent = getParent();
		if (parent instanceof TypeDefinition) {
			return (TypeDefinition) parent;
		}
		return null;
	}

}
