/*
 * mesopotamia-java @mesopotamia.version@
 * Multilingual parser and repository. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */

package org.mesopotamia.lang.java.javadoc;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.mesopotamia.MesopotamiaRuntimeException;

import antlr.collections.AST;

/**
 * @author Pavel Vlasov	
 * @version $Revision: 1.2 $
 */
public class Tag {
	private String name;
	private List<String> informalSpecification=new ArrayList<String>();
	private List<FormalSentence> formalSpecification=new ArrayList<FormalSentence>();
	private String informalText;
	
	public Tag(AST ast) {
		for (AST node=ast.getFirstChild(); node!=null; node=node.getNextSibling()) {
			switch (node.getType()) {
				case JavaDocTokenTypes.TAG:
					name=node.getFirstChild().getText();
					break;
				case JavaDocTokenTypes.INFORMAL:
					readInformalSpecification(node);
					break;
				case JavaDocTokenTypes.FORMAL:
					readFormalSpecification(node);
					break;
				default:
					throw new MesopotamiaRuntimeException("Invalid javadoc token type: "+ast.getType()); 
								
			}			
		}		
	}
	
	/**
	 * @param ast
	 */
	private void readInformalSpecification(AST ast) {
		StringBuffer buf=new StringBuffer();
		for (AST node=ast.getFirstChild(); node!=null; node=node.getNextSibling()) {
			informalSpecification.add(node.getText());
			if (buf.length()!=0) {
				buf.append(" ");
			}
			buf.append(node.getText());
		}
		informalText=buf.toString();
	}

	/**
	 * @param ast
	 */
	private void readFormalSpecification(AST ast) {
		for (AST node=ast.getFirstChild(); node!=null; node=node.getNextSibling()) {
			formalSpecification.add(new FormalSentence(node));
		}
	}

	public String getName() {
		return name;
	}
	
	public List<String> getInformalSpecification() {
		return Collections.unmodifiableList(informalSpecification);
	}

	public List<FormalSentence> getFormalSpecification() {
		return Collections.unmodifiableList(formalSpecification);
	}

	public String getInformalSpecificationAsString() {
		return informalText;
	}
}
