package org.mesopotamia.lang.java.ref.bcel;

import java.util.ArrayList;
import java.util.List;

import org.apache.bcel.classfile.ExceptionTable;
import org.apache.bcel.classfile.Method;
import org.apache.bcel.generic.Type;
import org.apache.bcel.util.Repository;
import org.mesopotamia.lang.java.ref.MethodInfo;
import org.mesopotamia.lang.java.ref.Scope;
import org.mesopotamia.lang.java.ref.TypeInfo;
import org.mesopotamia.lang.java.ref.TypeInfoImpl;
import org.mesopotamia.lang.java.ref.TypeSpecInfo;

public class JavaClassMethodInfo implements MethodInfo {
	
	private Method method;
	private JavaClassTypeInfoRepository repository;
	private TypeInfo declaringType;
	private Scope scope;

	JavaClassMethodInfo(Method method, Scope scope, JavaClassTypeInfoRepository repository, TypeInfo declaringType) {
		this.method = method;
		this.repository = repository;
		this.declaringType = declaringType;
		this.scope = scope;
	}
	

	public List<TypeInfo> getExceptionTypes() {
		List<TypeInfo> ret = new ArrayList<TypeInfo>();
		ExceptionTable et = method.getExceptionTable();
		if (et!=null) {
			String[] ena = et.getExceptionNames();
			for (int i=0; i<ena.length; ++i) {
				TypeInfo eType = repository.getTypeInfo(ena[i], scope);
				ret.add(eType==null ? new TypeInfoImpl(ena[i]) : eType);
			}
		}
		return ret;
	}

	public List<TypeSpecInfo> getParameterTypes() {
		List<TypeSpecInfo> ret = new ArrayList<TypeSpecInfo>();
		Type[] at = method.getArgumentTypes();
		for (int i=0; i<at.length; ++i) {
			ret.add(JavaClassTypeInfo.type2TypeSpecInfo(at[i], scope, repository));
		}		
		return ret;
	}

	public TypeSpecInfo getReturnType() {
		return JavaClassTypeInfo.type2TypeSpecInfo(method.getReturnType(), scope, repository);
	}

	public TypeInfo getDeclaringType() {
		return declaringType;
	}

	public String getFcn() {
		StringBuffer ret = new StringBuffer(declaringType.getFcn());
		ret.append(".");
		ret.append(method.getName());
		ret.append("(");
		Type[] at = method.getArgumentTypes();
		for (int i=0; i<at.length; ++i) {
			if (i>0) {
				ret.append(",");
			}
			ret.append(at[i].toString());
		}
		ret.append(")");
		return ret.toString();
	}

	public List<String> getModifiers() {
		return JavaClassTypeInfo.modifiersList(method.getModifiers());
	}

	public String getName() {
		return method.getName();
	}
	
	@Override
	public String toString() {
		return method.getSignature() + " ["+getClass().getName()+"]";
	}

}
