package org.mesopotamia.lang.java.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.sql.SQLException;

import biz.hammurapi.sql.SQLExceptionEx;
import biz.hammurapi.sql.SQLProcessor;
import biz.hammurapi.sql.Transaction;
import biz.hammurapi.sql.hsqldb.HsqldbServerDataSource;


/**
 * This utility class populates Mesopotamia table with Java-specific data.
 * @author Pavel Vlasov
 *
 */
public class InitDatabase implements Transaction {

	public boolean execute(SQLProcessor processor) throws SQLException {
		// Populate database
		String[] sqlResources= {
				// Set schema
				"org/mesopotamia/Init.sql",
				
				// Java 1.3 
				"org/mesopotamia/lang/java/v13/language.sql",
				"org/mesopotamia/lang/java/v13/tokenTypes.sql",
				"org/mesopotamia/lang/java/v13/tokenTypesUpdate.sql",
				"org/mesopotamia/lang/java/v13/loaders.sql",
				"org/mesopotamia/lang/java/v13/languageElementClass.sql",		
				
				// Java 1.4 
				"org/mesopotamia/lang/java/v14/language.sql",
				"org/mesopotamia/lang/java/v14/tokenTypes.sql",
				"org/mesopotamia/lang/java/v14/tokenTypesUpdate.sql",
				"org/mesopotamia/lang/java/v14/loaders.sql",
				"org/mesopotamia/lang/java/v14/languageElementClass.sql",		
				
				// Java 5 
				"org/mesopotamia/lang/java/v5/language.sql",
				"org/mesopotamia/lang/java/v5/tokenTypes.sql",
				"org/mesopotamia/lang/java/v5/tokenTypesUpdate.sql",
				"org/mesopotamia/lang/java/v5/loaders.sql",
				"org/mesopotamia/lang/java/v5/languageElementClass.sql"		
				
		};
		
		ClassLoader classLoader=InitDatabase.class.getClassLoader();
		for (int i=0; i<sqlResources.length; i++) {
			String resourceName = sqlResources[i];
			System.out.println("Loading "+resourceName);
			InputStream rStream = classLoader.getResourceAsStream(resourceName);
			if (rStream==null) {
				System.err.println("Resource not found: "+resourceName);
				return false;
			}
			Reader r=new InputStreamReader(rStream);
			try {
				try {
					processor.executeScript(r);
				} finally {
					r.close();
				}
			} catch (IOException e) {				
				throw new SQLExceptionEx("Cannot intialize database: "+e, e);
			}
		}	
		return true;
	}
	
	/**
	 * Connects to database and initializes it.
	 * Connects to HSQLDB on localhost with user sa and empty password.
	 * @param args Optional first parameter is database name.
	 * @throws SQLException 
	 * @throws ClassNotFoundException 
	 */
	public static void main(String[] args) throws Exception {
		System.out.println("Usage: java [options] org.mesopotamia.lang.java.util.InitDatabase ");
		// Create empty data source.
		HsqldbServerDataSource ds = new HsqldbServerDataSource(
				args.length==0 ? "localhost" : args[0], 
				args.length<2 ? "sa" : args[1], 
				args.length<3 ? "" : args[2], 
				null);
		
		SQLProcessor processor = new SQLProcessor(ds, null);
		processor.executeTransaction(new InitDatabase());
		ds.shutdown();
	}
}
